#include <stdio.h>
#include <cuda_runtime.h>

float* h_A;
float* h_B;
float* h_C;
float* d_A;
float* d_B;
float* d_C;

__global__ void VecAdd(const float* A, const float* B, float* C, int N) {
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    if (i<N)
       C[i] = A[i] + B[i];
}

void checkerror(cudaError_t err) {
    if(err) {
      fprintf(stderr, "CUDA Runtime API error %d: %s.\n",
      		      (int)err, cudaGetErrorString( err ) );
      exit(1);	   
    }
}

int main() {
    int N = 1000*1000;
    size_t size = N*sizeof(float);

    h_A = (float*)malloc(size);
    h_B = (float*)malloc(size);
    h_C = (float*)malloc(size);
    if(!h_A || !h_B || !h_C) {
      fprintf(stderr, "malloc failed\n");
      return 1;
    }
    cudaError_t err;
    err = cudaMalloc((void**)&d_A, size);
    checkerror(err);
    err = cudaMalloc((void**)&d_B, size);
    checkerror(err);
    err = cudaMalloc((void**)&d_C, size);
    checkerror(err);

    for(int c1=0;c1<N;c1++) {
        h_A[c1] = c1; // need random here
	h_B[c1] = h_A[c1]*h_A[c1];
    }

    err = cudaMemcpy(d_A, h_A, size, cudaMemcpyHostToDevice);
    checkerror(err);
    err = cudaMemcpy(d_B, h_B, size, cudaMemcpyHostToDevice);
    checkerror(err);

    int threadsPerBlock = 256;
    int blocksPerGrid = (N + threadsPerBlock - 1) / threadsPerBlock;
    VecAdd<<<blocksPerGrid, threadsPerBlock>>>(d_A, d_B, d_C, N);
    err=cudaGetLastError();
    checkerror(err);

    err = cudaDeviceSynchronize(); // cudaMemcpy is anyway a blocking call
    checkerror(err);

    err = cudaMemcpy(h_C, d_C, size, cudaMemcpyDeviceToHost);
    checkerror(err);

    for(int c1=N/2;c1<N&&c1<N/2+10;c1++) {
    	 printf("%f %f %f\n",h_A[c1], h_B[c1], h_C[c1]); 
    }


    cudaFree(d_A);
    cudaFree(d_B);
    cudaFree(d_C);
    free(h_A);
    free(h_B);
    free(h_C);

    printf("Finished\n");
    return 0;
}
